#include <GL/glew.h>
#include "CI/CISG.h"
#include "CI/CICubeMap.h"
#include "CI/CIMsg.h"
#include "MMGrid.h"
#include "MMSG.h"

using namespace vcg ;
using namespace std ;

void CICubeMapNode::XMLWrite(QTextStream &xstrm)
{
	xstrm << "<CICubeMapNode>\n";
	xstrm << textureBaseName ;
	xstrm << "</CICubeMapNode>\n";
}

bool CICubeMapNode::XMLRead(QDomElement n, CISG *) 
{
	if(n.tagName() !="CICubeMapNode" || !n.firstChild().isText() ) 
		return false;
	qDebug("< %s > %s ...\n",n.tagName().ascii(),n.text().ascii());
	QTextIStream(&n.text()) >> textureBaseName;
  return true;
}

void CICubeMapNode::drawGL(const float )
{
	if(dirty) initializeGL();
	CIMsg::checkGLError("drawGL");
	// estrae la componente rotazione dalla modelview corrente.
	Matrix44f mm;
	glGetModelViewMatrix(mm);
	mm.a[0][3]=0;
	mm.a[1][3]=0;
	mm.a[2][3]=0;
	mm.a[3][3]=1;
	float dt=1.0f/pow(mm.Det(),1.0f/3.0f);
	Matrix44f ss; ss.Scale(dt,dt,dt);
	rr=mm*ss;
	dt=rr.Det();
	DrawEnvCube(rr);
}


void CIGridNode::XMLWrite(QTextStream &xstrm)
{
	xstrm << "<CIGridNode sx=\"" << sx << "\" sy=\"" << sy << "\" lateraltexFileName=\"" << lateraltexFileName << "\" exist=\"" << bumper.exist << "\" resping=\"" << bumper.resping << "\" begin0=\"" << bumper.begin[0] << "\" begin1=\"" << bumper.begin[1]<< "\" begin2=\"" << bumper.begin[2] << "\" end0=\"" << bumper.end[0]<< "\" end1=\"" << bumper.end[1]<< "\" end2=\"" << bumper.end[2] << "\" start0=\"" << start.p[0] << "\" start1=\"" << start.p[1]<< "\" start2=\"" << start.p[2] << "\" leveltime=\"" << leveltime << "\">" << " \n";
	vector<MMCell>::iterator ii;
	for(ii=g.begin();ii!=g.end();++ii)
		(*ii).XMLWrite(xstrm);
	xstrm << "</CIGridNode>\n";
}

bool CIGridNode::XMLRead(QDomElement en, CISG *) 
{
	if(en.tagName() !="CIGridNode" || !en.firstChild().isText() )
		return false;
	qDebug("< %s > %s ...\n",en.tagName().ascii(),en.text().ascii());
	sx = en.attribute("sx","10").toInt();
	sy = en.attribute("sy","10").toInt();
		resize(sx,sy);
	lateraltexFileName = en.attribute("lateraltexFileName");
	//glue = en.attribute("glue").toInt();
	//ice = en.attribute("ice").toInt();
	bumper.exist=en.attribute("exist").toInt();
	bumper.resping=en.attribute("resping").toInt();
	bumper.begin[0]=en.attribute("begin0").toFloat();
	bumper.begin[1]=en.attribute("begin1").toFloat();
	bumper.begin[2]=en.attribute("begin2").toFloat();
	bumper.end[0]=en.attribute("end0").toFloat();
	bumper.end[1]=en.attribute("end1").toFloat();
	bumper.end[2]=en.attribute("end2").toFloat();
	start.p[0]=en.attribute("start0").toFloat();
	start.p[1]=en.attribute("start1").toFloat();
	start.p[2]=en.attribute("start2").toFloat();
	leveltime=en.attribute("leveltime").toInt();
		


	vector<MMCell>::iterator ii;
	QTextIStream is(&en.text());
	for(ii=g.begin();ii!=g.end();++ii)
		(*ii).XMLRead(is);
	generateFace();
	return true;
}


CISGNode *MMSG::Allocate(const QString &classname)
{
	CISGNode *ret=CISG::Allocate(classname);		
	if(classname=="CIGridNode") {
		g=new CIGridNode();
		ret=g;
		}
	if(classname=="CICubeMapNode") {
		c=new CICubeMapNode();
		ret=c;
		}
	
	return ret;
}	


